@@ -8,6 +8,7 @@ from pymoodle_jku.Classes.evaluation import Evaluation
 from pymoodle_jku.Client.client import MoodleClient
 from pymoodle_jku.Client.download_manager import DownloadManager
 from pymoodle_jku.Utils.config import config
+from pymoodle_jku.Utils.login import relogin
 from pymoodle_jku.Utils.printing import print_pick_results_table
 
 logger = logging.getLogger(__name__)
@@ -29,9 +30,9 @@ def get_all_downloads(dir_: Path, links: List[Union[Url, Evaluation]]) -> Tuple[
     try:
         urls = url_list.read_text().splitlines()
         f = list(filter(lambda l: (l.link if type(l) is Url else l.url) not in urls, links))
-        return (f, urls)
-    except:
-        return (links, [])
+        return f, urls
+    except FileNotFoundError:
+        return links, []
 
 
 def write_urls(dir_: Path, urls: List[str]) -> None:
@@ -45,6 +46,7 @@ def write_urls(dir_: Path, urls: List[str]) -> None:
     url_list.write_text('\n'.join(urls))
 
 
+@relogin
 def main(client: MoodleClient, args):
     path = args.path or config['Path']
     path = Path('./' if path is None else path)
@@ -92,7 +94,7 @@ def main(client: MoodleClient, args):
         cur_dir = path / (c.parse_name())
         try:
             cur_dir.mkdir()
-        except:
+        except (FileNotFoundError, OSError):
             pass
 
         valuations = client.single_valuation(c)
@@ -124,7 +126,4 @@ if __name__ == "__main__":
     # The folder must exist already
     # and the path must be relative to where you start the process
     # or you can specify an absolut path
-    try:
-        pass
-    except Exception as err:
-        debug(str(err))
+    pass
