@@ -6,6 +6,11 @@ from typing import Optional, Any, List
 from pick import Picker
 
 
+def print_exc(exception):
+    print('Please Notify LeLunZ on the JKU Discord Channel:')
+    print(str(exception))
+
+
 def clean_screen():
     """
     Clears the terminal screen.
@@ -37,7 +42,7 @@ def print_courses(client):
     print(output, flush=True)
 
 
-def print_pick_results_table(data: List, multiselect=False) -> Optional[Any]:
+def print_pick_results_table(data: List, multiselect=False, load_more=True) -> Optional[Any]:
     """
     Prints a interactive table.
     :param data: The data that should be printed.
@@ -58,10 +63,12 @@ def print_pick_results_table(data: List, multiselect=False) -> Optional[Any]:
         items.append(line_out)
 
     if multiselect:
-        header = 'Press [Spacebar] to select and [Enter] to continue. [q] to exit. [m] to load more.'
+        header = 'Press [Space] to select and [Enter] to continue. [q] to exit.' + (
+            ' [m] to load more from old semesters.' if load_more else '')
         picker = Picker(items, header, multiselect=True, min_selection_count=1)
     else:
-        header = 'Press [Enter] to continue. [q] to exit. [m] to load more.'
+        header = 'Press [Enter] to continue. [q] to exit.' + (
+            ' Press [m] to load more from old semesters.' if load_more else '')
         picker = Picker(items, header)
 
     def return_exit(_):
